#include "wav_loader.h"
#include "argv.h"
#include "cvt.h"
#include "adpcm.h"
#include <stdio.h>
#include <stdlib.h>

static char lib_buffer[ 0x1000 ];

struct AdpcmFile {
	int header;
	unsigned short samplerate;
	unsigned short channel;
	int length;
	int loopPosition;
};

int main( int argc, char* argv[] ){

	char loadFilename[ 0x100 ];
	char* saveFilename;
	struct PcmFormat sample;

	lib_initResourcename( lib_buffer, 0x1000 );

	if( argc != 2 ){
		printf( "Error : No input file\n" );
		printf( "usage : adpcm_converter <input file name>\n" );
		return 1;
	}

	lib_strcpy( loadFilename, argv[ 1 ] );
	saveFilename = lib_changeExtensiton( loadFilename, ".adpcm" );

	sample.sampleptr = NULL;
	if( loadwave( loadFilename, &sample ) == 1 ){
		int _writeAdpcmFile = 0;
		int guess[ 2 ] = { YM2608_DEFAULT_GUESS, YM2608_DEFAULT_GUESS };
		int delta[ 2 ] = { YM2608_DEFAULT_DELTA, YM2608_DEFAULT_DELTA };
		struct AdpcmFile _adpcmFile;
		int tempLength = ( sample.soundlen / 4 ) & 0x7FFFFFF0; /* PCM 16bit -> ADPCM 4bit */
		unsigned char* tempData = ( unsigned char* )malloc( tempLength );

		/* `FbN */
		if(( sample.channels != 1 ) && ( sample.channels != 2 )){
			printf( "ERROR : Not support file.\n" );
			printf( "issue : support 1ch and 2ch only.\n" );
			return 1;
		}
		/* `FbN */
		if(( sample.frequency != 11025 ) && ( sample.frequency != 22050 ) && ( sample.frequency != 44100 )){
			printf( "ERROR : Not support file.\n" );
			printf( "issue : support 11025Hz , 22050Hz and 44100Hz only.\n" );
			return 1;
		}
		/* `FbN */
		if( sample.bits != 16 ){
			printf( "ERROR : Not support file.\n" );
			printf( "issue : support 16bits only.\n" );
			return 1;
		}

		_adpcmFile.header = 0;
		_adpcmFile.channel = sample.channels;
		_adpcmFile.length = tempLength * ( sample.channels == 1 ? 2 : 1 );
		_adpcmFile.samplerate = sample.frequency;
		_adpcmFile.loopPosition = tempLength * ( sample.channels == 1 ? 2 : 1 );

		/* wav t@Cm`lȂ΁Am adpcm GR[h */
		if( sample.channels == 1 ){
			ym2608adpcm_encode1( ( short* )sample.sampleptr, tempData, tempLength * 2, guess, delta );
			/* t@Co͏̎s\ */
			_writeAdpcmFile = 1;
		}
		/* wav t@CXeI`lȂ΁AXeI adpcm GR[h */
		if( sample.channels == 2 ){
			ym2608adpcm_encode2( ( short* )sample.sampleptr, tempData, tempLength, guess, delta );
			/* t@Co͏̎s\ */
			_writeAdpcmFile = 1;
		}
		if( _writeAdpcmFile ){
			FILE* fp;
			fp = fopen( saveFilename, "wb" );
			if( fp ){
				fwrite( &_adpcmFile, sizeof( struct AdpcmFile ), 1, fp );
				fwrite( tempData, tempLength, 1, fp );
				fclose( fp );
			}
		}
	}else{
		printf( "ERROR : Not support file.\n" );
		printf( "issue : support PCM format only.\n" );
		return 1;
	}

	printf( "Encode success.\n" );
	return 0;
}

