#include "wav_loader.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define HEX_RIFF  ( 0x46464952 )
#define HEX_WAVE  ( 0x45564157 )
#define HEX_fmt   ( 0x20746D66 )
#define HEX_data  ( 0x61746164 )

#define FMT_PCM   ( 0x0001 )

int loadwave( char *filename, struct PcmFormat *buf ){
	struct{
		unsigned int riff;
		unsigned int size;
		unsigned int type;
	}riffheader;
	struct{
		unsigned int tag;
		unsigned int size;
	}rifftag;
	struct{
		unsigned short format;     /* 1 = PCM */
		unsigned short channels;   /* Mono, stereo */
		unsigned int samplerate;   /* 11025, 22050, 44100 */
		unsigned int bps;          /* Bytes/second */
		unsigned short unknown;
		unsigned short samplebits; /* 8, 12, 16 */
	}fmt;
	
	FILE *fp;
	
	if(buf == NULL){
		return 0;
	}
	
	fp = fopen( filename, "rb" );
	if( !fp ){
		printf( "File not found. [file=%s]\n", filename );
		return 0;
	}
	if( fread( &riffheader, sizeof( riffheader ), 1, fp ) != 1 ){
		printf( "Riff header error.\n" );
		return 0;
	}
	if( ( riffheader.riff != HEX_RIFF ) || ( riffheader.type != HEX_WAVE ) ){
		printf( "Header error.\n" );
		return 0;
	}
	
	rifftag.tag = 0;
	
	/* Search for format tag */
	while( rifftag.tag!=HEX_fmt ){
		if( fread( &rifftag, sizeof( rifftag ), 1, fp ) != 1 ){
			printf( "Read error. Tag reading.\n" );
			return 0;
		}
		if( rifftag.tag != HEX_fmt ){
			printf( "Seek change.\n" );
			fseek( fp, rifftag.size, SEEK_CUR );
		}
	}
	if( fread( &fmt, sizeof( fmt ), 1, fp ) != 1 ){
		return 0;
	}
	if( rifftag.size > sizeof( fmt )){
		fseek( fp, rifftag.size - sizeof( fmt ), SEEK_CUR);
		printf( "fseek=%u\n", rifftag.size - sizeof( fmt ) );
	}
	if( fmt.format != FMT_PCM || ( fmt.channels != 1 && fmt.channels != 2 ) || ( fmt.samplebits != 8 && fmt.samplebits != 16 )){
		printf( "Header error.\n" );
		return 0;
	}
	
	/* Search for data tag */
	while(rifftag.tag!=HEX_data){
		if( fread( &rifftag, sizeof( rifftag ), 1, fp ) != 1 ){
			printf( "Not found of data tag.\n" );
			return 0;
		}
		if( rifftag.tag != HEX_data ){
			fseek( fp, rifftag.size, SEEK_CUR );
		}
	}

	buf->sampleptr = ( char* )malloc( rifftag.size + 8 );
	if( buf->sampleptr == NULL ){
		printf( "Out of memory.\n" );
		return 0;
	}
	if( fmt.samplebits == 8 ){
		memset( buf->sampleptr, 0x80, rifftag.size + 8 );
	}else{
		memset( buf->sampleptr, 0, rifftag.size + 8 );
	}

	if( fread( buf->sampleptr, rifftag.size, 1, fp ) != 1 ){
		free( buf->sampleptr );
		printf( "Data read error. Bad data length.\n" );
		return 0;
	}

	fclose( fp );

	buf->soundlen = rifftag.size;
	buf->bits = fmt.samplebits;
	buf->frequency = fmt.samplerate;
	buf->channels = fmt.channels;
	
	return 1;
}

