#include "fileload.h"
#include "cvt.h"
#include <cstdio>
#include <cstdlib>
#include <SDL.h>

using namespace Utility;

FileLoad::FileLoad(){
	for( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		_data[ i ].ptr = 0;
		_data[ i ].type = TYPE_NUM;
		_data[ i ].tag[ 0 ] = '\0';
	}
	_valid = true;
}

FileLoad::~FileLoad(){
	for( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if( _data[ i ].type == TYPE_NORMAL ){
			free( _data[ i ].ptr );
		}
		if( _data[ i ].type == TYPE_SDL_SURFACE ){
			SDL_FreeSurface( ( SDL_Surface* )_data[ i ].ptr );
		}
	}
}

bool FileLoad::load( const char* filename, const char* tag, int type ){
	int id = 0;
	for( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		id = i;
		if( _data[ i ].ptr == 0 ){
			if( type == TYPE_NORMAL ){
				FILE* fp = 0;
				int length;
				char* ptr = 0;

				fp = fopen( filename, "rb" );
				if( !fp ){
					return false;
				}
				fseek( fp, 0, SEEK_END );
				length = ftell( fp );
				fseek( fp, 0, SEEK_SET );

				ptr = new char[ length ];

				fread( ptr, 1, length, fp );

				fclose( fp );

				_data[ id ].ptr = ptr;
				lib_strcpy( _data[ id ].tag, tag );
				_data[ id ].length = length;
				_data[ id ].type = TYPE_NORMAL;

			}else{
				_data[ id ].ptr = reinterpret_cast< char* >( SDL_LoadBMP( filename ) );
				if( !_data[ id ].ptr ){
					return false;
				}
				_data[ id ].type = TYPE_SDL_SURFACE;
				lib_strcpy( _data[ id ].tag, tag );
			}
			return true;
		}
	}
	_valid = false;
	return false;
}

void FileLoad::release(){
	for( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if( _data[ i ].ptr ){
			free( _data[ i ].ptr );
			_data[ i ].ptr = 0;
			_data[ i ].tag[ 0 ] = '\0';
		}
	}
}


char* FileLoad::getPtr( char* tag ){
	for( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if( lib_strcmp( _data[ i ].tag, tag ) == 0 ){
			return _data[ i ].ptr;
		}
	}
	return 0;
}

int FileLoad::getLength( char* tag ){
	for( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if( lib_strcmp( _data[ i ].tag, tag ) == 0 ){
			return _data[ i ].length;
		}
	}
	return 0;
}

SDL_Surface* FileLoad::getSurface( char* tag ){
	for( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if( lib_strcmp( _data[ i ].tag, tag ) == 0 ){
			return reinterpret_cast< SDL_Surface* >( _data[ i ].ptr );
		}
	}
	return 0;
}

