#include <SDL.h>

// Timer ----------------------------------------------------------------------
#include "timer.h"

// Keyboard -------------------------------------------------------------------
#include "keyboard.h"

// Sound ----------------------------------------------------------------------
#include "sound_core.h"
#include "sound_adpcm.h"

// Common ---------------------------------------------------------------------
#include "cvt.h"
#include "fileload.h"

namespace {
	struct AdpcmFile {
		int header;
		unsigned short samplerate;
		unsigned short channel;
		int length;
		int loopPosition;
	};

	const int VIEW_WIDTH = 640;
	const int VIEW_HEIGHT = 240;
	const int VIEW_COLOR = 32;
	const int VIEW_TYPE = SDL_SWSURFACE;
	char* SOUND_ID = "ADPCM";
	int _deviceSampleRate = 44100;
	int _deviceSoundBuffer = 1024;
}

// SDL_main -------------------------------------------------------------------
int main( int argc, char *argv[] ){

	Device::TimerSDL timer;
	Device::Input::KeyboardSDL key;
	Driver::BGM::Adpcm _adpcm;
	Utility::FileLoad _file;

	int thread = 1;
	char str[ 0x100 ] = { 0 };
	int volume = SOUND_GAME_VOLUME;
	
	if( argc != 2 ){
		printf( "Error : No input file\n" );
		printf( "usage : adpcm_player <input adpcm file>\n" );
		return 1;
	}

	_file.load( argv[ 1 ], "adpcm" );

	if( !_file.getPtr( "adpcm" ) ){
		printf( "adpcm not found.\n" );
		return 0;
	}
	printf( "playing argv[1]=%s .\n", argv[ 1 ] );

	// (1) SDL Cu֘A
	SDL_Surface *primary = NULL;
	SDL_Event event;
	
	// (1) SDL Cu
	if( SDL_Init( SDL_INIT_EVERYTHING ) < 0 ) {
		fprintf( stderr ,"SDL_Init() failed.\n" );
		fprintf( stderr, "%s\n", SDL_GetError() );
		exit( 1 );
	}

	// (1) ʂ
	primary = SDL_SetVideoMode( VIEW_WIDTH, VIEW_HEIGHT, VIEW_COLOR, VIEW_TYPE );
	if( primary == NULL ) {
		fprintf( stderr, "SDL_SetVideoMode() failed.\n" );
		fprintf( stderr, "%s\n", SDL_GetError() );
		exit( 1 );
	}

	// (3) TEh֘A
	_adpcm.initialize();

	// (3-1) TEhf[^̓o^
	{
		unsigned char* ptr = reinterpret_cast< unsigned char* >( _file.getPtr( "adpcm" ) );
		struct AdpcmFile _adpcmFile;
		lib_memcpy( &_adpcmFile, ptr, 0x10 );
		_adpcm.entry( SOUND_ID,
			ptr + 0x10,
			_adpcmFile.length,
			_adpcmFile.samplerate,
			_adpcmFile.channel,
			_adpcmFile.length );
	}
	
	// (3-2) TEhfoCX̏
	_adpcm.setFillFormat( _deviceSampleRate, 2 );
	SDL_SoundSetCallback( 0, &_adpcm );
 
	SDL_SoundOpenDevice( _deviceSampleRate, _deviceSoundBuffer, 2 );

	timer.setFPS( 60 );
	timer.reset();

	_adpcm.play( SOUND_ID );

	while( thread ){

		// EChECxg
		SDL_PollEvent( &event );
		key.update();

		if( key.count( SDLK_ESCAPE ) == 1 ){
			thread = 0;
		}
		if( key.count( SDLK_SPACE ) == 1 ){
			_adpcm.play( SOUND_ID );
			_adpcm.setGameVolume( volume );
		}
		if( key.count( SDLK_RETURN ) == 1 ){
			_adpcm.play( SOUND_ID );
			_adpcm.setGameVolume( volume );
			_adpcm.setEndBgm5sec();
		}
		
		switch( event.type ){
		case SDL_QUIT:
			/* Set whatever flags are necessary to */
			/* end the main game loop here */
			thread = 0;
			break;
		}
		
		sprintf( str, "AdpcmPlayer Playing %4d.%.2dsec(%8dsamples)",
			_adpcm.getPlayedBgmTimes() / 100,
			_adpcm.getPlayedBgmTimes() % 100,
			_adpcm.getPlayedBgmSamples()
			);

		SDL_WM_SetCaption( str, 0 );

		if( !timer.isFrameSkip() ){
			// `œhԂ
			if( SDL_FillRect( primary, NULL, 0 ) != 0 ){
				// This function returns 0 on success, or -1 on error.
				fprintf( stderr, "%s\n", SDL_GetError() );
				exit( 1 );
			}

			// `揈͂ɓ

			// ]
			SDL_UpdateRect( primary, 0, 0, VIEW_WIDTH, VIEW_HEIGHT );
		}
		// wait
		timer.wait();
	}
	return 0;
}
