#ifndef __sound_adpcm__
#define __sound_adpcm__

#include "sound_core.h"

namespace Driver {
	namespace BGM {

		struct SoundSlot {
			int len;
			int samplerate;			// 11025/22050/44100
			int channel;			// 1/2
			int compress;			// Adpcm/Pcm/PSG/SCC
			unsigned char* buf;		// Data
			int id;					// Ȕԍtp

			int play;
			int volume[ 2 ];		// 0-16
			int pos;
			int playedPos;
			int playedPosL;
			int guess[ 2 ];			// Adpcm ̃\[X
			int delta[ 2 ];			// Adpcm ̃\[X

			int posLoop;			// XyV[vp
			int guessLoop[ 2 ];		// XyV[vp
			int deltaLoop[ 2 ];		// XyV[vp
		};

		struct AdpcmList {
			int len;
			int samplerate;			// 11025/22050/44100
			int channel;			// 1/2
			int compress;			// Adpcm/Pcm/PSG/SCC
			unsigned char* buf;		// Data

			int posLoop;			// XyV[vp   -1 ܂ 0x7FFFFFFF ̏ꍇ́AɂȂĂ
			int guessLoop[ 2 ];		// XyV[vp
			int deltaLoop[ 2 ];		// XyV[vp
		};

		struct SoundNameList {
			int id;
			char name[ 0x40 ];
		};

		const int MAX_ID_NUM = 0x80;

		class Adpcm : public SoundInterface {
		public:
			Adpcm();
			virtual ~Adpcm();

			void initialize( void );

			void fill( int* stream, int length );
			void setFillFormat( int samplerate, int channel );

			void decode( struct SoundSlot*, int*, int, int* );

			int entry( char* name, unsigned char* buf, int len, int samplerate, int channel, int loopPos );
			int setSpecialLoop( const char* name, int pos );
			int erase( char* name );

			int play( const char* );
			void stop( void );

			void setUserVolume( int );
			int getUserVolume( void );

			void setGameVolume( int );
			int getGameVolume( void );

			int getId( const char* );

			int getPlayedBgmTimes( void );
			int getPlayedBgmSamples( void );

			void setEndBgm5sec( void );

		protected:
			AdpcmList _list[ MAX_ID_NUM ];
			SoundNameList _cvt[ MAX_ID_NUM ];
			SoundSlot _slot;
			int _userVolume;			// 0-16  [Uݒ肷鉹
			int _samplerate;			// output 11025 | 22050 | 44100
			int _channel;				// output 1 | 2
			int _mutex;
			int* resampleBuffer;
		};
	}
}

#endif // __sound_adpcm__
