#include "sound_core.h"
#include "cvt.h"
#include <SDL.h>

namespace {
	SDL_AudioSpec *desired, *obtained;

	struct SoundFillCount soundFillCount;

	const int SOUND_MAX_ID = 4;
	SoundInterface* _callback[ SOUND_MAX_ID ];
	int pcmBuffer[ 2048 ];
	int SDOUND_CHANNEL = 2;

	int _enable;
}

extern "C" {

	int SDL_SoundSetCallback( int id, SoundInterface* sndInterface ){
		if( id > SOUND_MAX_ID || id < 0 ){
			return 0;
		}
		_callback[ id ] = sndInterface;
		return 1;
	}

	void SDL_SoundClearCallback( void ){
		for( int id = 0; id < SOUND_MAX_ID; ++id ){
			_callback[ id ] = 0;
		}
	}

	void SDL_SoundFill( void* udata, unsigned char* stream, int length ){
		int i;
		short* stream16 = reinterpret_cast< short* >( stream );
		
		if( !_enable ){
			return;
		}

		lib_memset( pcmBuffer, 0, length * 2 );

		for( i = 0; i < SOUND_MAX_ID; ++i ){
			if( _callback[ i ] ){
				_callback[ i ]->fill( pcmBuffer, length / ( 2 * SDOUND_CHANNEL ) );
			}
		}

		for( i = 0; i < length / 2; ++i ){
			stream16[ i ] = lib_movs_sword( pcmBuffer[ i ] >> 12 );
		}

		// warning: unused ΍
		udata = 0;
	}

	bool SDL_SoundOpenDevice( int input_audio_samplerate, int sound_buffer, int channel ){

		desired = ( SDL_AudioSpec* ) malloc( sizeof( SDL_AudioSpec ) );

		obtained = ( SDL_AudioSpec* ) malloc( sizeof( SDL_AudioSpec ) );

		desired->freq = input_audio_samplerate;
		desired->format = AUDIO_S16;

		desired->channels = SDOUND_CHANNEL = ( channel == 2 ) ? 2 : 1;
		desired->samples = sound_buffer;

		desired->callback = SDL_SoundFill;
		desired->userdata = 0;

		if ( SDL_OpenAudio( desired, obtained ) < 0 ) {
			fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
			return false;
		}
		SDL_PauseAudio( 0 );

		free( desired );
		
		_enable = 1;

		return true;
	}
	
	void SDL_SoundCloseDevice( void ){
		SDL_CloseAudio();
		free( obtained );
		_enable = 0;
	}

	void SDL_SoundEnableSoundDevice(){
		_enable = 1;
	}

	void SDL_SoundDisableSoundDevice(){
		_enable = 0;
	}

}
